{
 TPopupButton v1.2

 Assign a popupbutton to to TPopupMenu property and
 you're off!!

 First:
 Thanks to Gabriel Cherem for finding a bug in the 1.0 release.
 If you delete the linked popup component before the popupbutton
 (at design time), Delphi's IDE crashes (GPF) because the PopupMenu
 property will point to a component that no longer exists.
 Thanks for that one ;)

 Second:
 Modified the source for use with delphi 3.0
 Added the Charset property in the PaintCaption method
 for proper use of the Marlett font (down arrow)

 Third:
 Added a shortcut key handler.
 The button can now be activated through the
 select shortcut character in the caption.


 Comments/Suggestions?
 Write mail to: m.nieuwhof@hs-ipabo.edu
}
unit PopBut;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtCtrls;

Const
   btNormal   = 0;
   btDown     = 1;
   btFocus    = 2;
   btDisabled = 3;

type
  TPopupButton = class(TCustomControl)
  private
    { Private declarations }
    Rc:         TRect;
    MDown:      Boolean;
    FPopupMenu: TPopupmenu;
    // Private procedures
    Procedure CMFontChanged(var Msg: TMessage); message CM_FONTCHANGED;
    Procedure CMEnabledChanged(var Msg: TMessage); message CM_ENABLEDCHANGED;
    Procedure WMSetFocus(Var Message: TWMSetFocus); message WM_SETFOCUS;
    Procedure WMKillFocus(Var Message: TWMKillFocus); message WM_KILLFOCUS;
    Procedure CMTextChanged(var Msg: TMessage); message CM_TEXTCHANGED;
    Procedure CMDialogChar(var Msg: TCMDialogChar); message CM_DIALOGCHAR;
    // My Own functions
    Procedure PaintButton (Mode: Integer);
    Procedure PaintCaption(Mode: Integer);
  protected
    { Protected declarations }
  public
    { Public declarations }
    Constructor Create (AOwner: TComponent); override;
    Procedure   Paint; override;
    Procedure   Click; override;
    Procedure   MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    Procedure   KeyPress(var Key: Char); override;
    Procedure   Notification (AComponent: TComponent; Operation: TOperation); override;
  published
    { Published declarations }
    Property PopupMenu: TPopupMenu
             Read       FPopupMenu
             Write      FPopupMenu;
    Property Align;
    Property Caption;
    Property Enabled;
    Property Font;
    Property ShowHint;
    Property TabOrder;
    Property TabStop;
    Property Visible;
    // Events
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
End;

procedure Register;

implementation
//
// Register the component on the palette
//
procedure Register;
begin
  RegisterComponents('Win32', [TPopupButton]);
end;
//
// Create the component
//
Constructor TPopupButton.Create(AOwner: TComponent);
Begin
    Inherited Create(AOwner);
    ControlStyle := [csCaptureMouse,
                     csClickEvents,
                     csSetCaption,
                     csDoubleClicks];
    Width:=120;
    Height:=23;
    TabStop:=True;
    MDown:=False;
    If Assigned(PopupMenu) then
        PopupMenu.AutoPopup:=False;
End;
//
// Paint the button, choose from three states!
//
Procedure TPopupButton.PaintButton(Mode: Integer);
Var
    Brc: TRect;
    Pa:  Array[0..2] of TPoint;
Begin
    Rc:=GetClientRect;
    Brc:=Rc;
    With Canvas do begin
        Case mode of
        btNormal: Begin
                  // Clear clientarea
                  Brush.Color:=clBlack;
                  Brush.Style:=bsSolid;
                  FillRect(Rc);
                  // Draw button unfocused, normal!
                  Brush.Color:=clBtnFace;
                  Rectangle(Rc.Left-1,Rc.Top-1,Rc.Right-1,Rc.Bottom-1);
                  Pen.Color:=clBtnHighlight;
                  MoveTo(0,0);
                  LineTo(0,Height-1);
                  MoveTo(0,0);
                  LineTo(Width-1,0);
                  Pen.Color:=cl3dLight;
                  MoveTo(1,1);
                  LineTo(1,Height-2);
                  MoveTo(1,1);
                  LineTo(Width-2,1);
                  Pen.Color:=clBtnShadow;
                  MoveTo(1,Height-2);
                  LineTo(Width-1,Height-2);
                  MoveTo(Width-2,1);
                  LineTo(Width-2,Height-2);
                  End;
        btFocus:  Begin
                  // Clear clientarea
                  Brush.Color:=clBlack;
                  Brush.Style:=bsSolid;
                  FillRect(Rc);
                  // Draw button focused
                  Brush.Color:=clBtnFace;
                  Rectangle(Rc.Left+1,Rc.Top+1,Rc.Right-2,Rc.Bottom-2);
                  Pen.Color:=clBtnHighlight;
                  MoveTo(1,1);
                  LineTo(1,Height-2);
                  MoveTo(1,1);
                  LineTo(Width-2,1);
                  Pen.Color:=cl3dLight;
                  MoveTo(2,2);
                  LineTo(2,Height-2);
                  MoveTo(2,2);
                  LineTo(Width-2,2);
                  Pen.Color:=clBtnShadow;
                  MoveTo(2,Height-3);
                  LineTo(Width-3,Height-3);
                  MoveTo(Width-3,Height-3);
                  LineTo(Width-3,1);
                  Rc.Left:=4;
                  Rc.Top:=4;
                  Rc.Right:=Rc.Right-4;
                  Rc.Bottom:=Rc.Bottom-4;
                  DrawFocusRect(Rc);
                  End;
        btDown:   Begin
                  // Clear clientarea
                  Brush.Color:=clBlack;
                  Brush.Style:=bsSolid;
                  FillRect(Rc);
                  // Draw button when downed!
                  Brush.Color:=clBtnFace;
                  Rectangle(Rc.Left+1,Rc.Top+1,Rc.Right-1,Rc.Bottom-1);
                  Pen.Color:=clBtnShadow;
                  Brush.Style:=bsClear;
                  Rectangle(Rc.Left+1,Rc.Top+1,Rc.Right-1,Rc.Bottom-1);
                  Brush.Style:=bsSolid;
                  Rc.Left:=4;
                  Rc.Top:=4;
                  Rc.Right:=Rc.Right-4;
                  Rc.Bottom:=Rc.Bottom-4;
                  DrawFocusRect(Rc);
                  End;
        End;
        If Enabled then
            PaintCaption(Mode)
        Else
            PaintCaption(btDisabled);
    End;
End;
//
// Paint the caption
//
Procedure TPopupButton.PaintCaption(Mode: Integer);
Var
    Bf: TFont;
Begin
    With Canvas Do Begin
        Bf:=TFont.Create;
        Bf.Name:=Font.Name;
        Bf.Size:=Font.Size;
        Bf.Style:=Font.Style;
        Bf.Charset:=Font.Charset;
        Font.Color:=clBtnText;
        If Mode<>BtDisabled then Begin
            // Backup canvas fontdata
            Rc:=GetClientRect;
            Brush.Style:=bsClear;
            // Draw the caption
            If (Mode=btNormal) or (Mode=btFocus) then begin
                Rc.Left:=6;
                Rc.Top:=((Height-TextHeight('0g')) div 2)-1;
                DrawText(Handle,Pchar(Caption),Length(Caption),Rc,DT_LEFT);
                End
            Else Begin
                Rc.Left:=7;
                Rc.Top:=((Height-TextHeight('0g')) div 2);
                DrawText(Handle,Pchar(Caption),Length(Caption),Rc,DT_LEFT);
            End;
            // Draw the DOWN typeface
            Font.Name:='Marlett';
            Font.Size:=Font.Size+4;
            Font.Charset:=SYMBOL_CHARSET;
            If (Mode=btNormal) or (Mode=btFocus) then begin
                Rc.Left:=Width-20;
                Rc.Top:=((Height-TextHeight('0')) div 2);
                DrawText(Handle,'6',1,Rc,DT_LEFT);
                End
            Else Begin
                Rc.Left:=Width-19;
                Rc.Top:=((Height-TextHeight('0')) div 2)+1;
                DrawText(Handle,'6',1,Rc,DT_LEFT);
            End;
        End
        Else Begin
            // Draw the disabled type face
            Brush.Style:=bsClear;
            Rc:=GetClientRect;
            Rc.Left:=6;
            Rc.Top:=((Height-TextHeight('0g')) div 2)-1;
            Font.Color:=clBtnHighLight;
            OffsetRect(Rc,1,1);
            DrawText(Handle,PChar(Caption),Length(Caption),Rc,DT_LEFT);
            Font.Color:=clGrayText;
            OffsetRect(Rc,-1,-1);
            DrawText(Handle,PChar(Caption),Length(Caption),Rc,DT_LEFT);

            Font.Name:='Marlett';
            Font.Size:=Font.Size+4;
            Rc:=GetClientRect;
            Rc.Left:=Width-20;
            Rc.Top:=((Height-TextHeight('0')) div 2);
            Font.Color:=clBtnHighLight;
            OffsetRect(Rc,1,1);
            DrawText(Handle,'6',1,Rc,DT_LEFT);
            Font.Color:=clGrayText;
            OffsetRect(Rc,-1,-1);
            DrawText(Handle,'6',1,Rc,DT_LEFT);
        End;
        // Restore canvas fontdata
        Font.Name:=Bf.Name;
        Font.Size:=Bf.Size;
        Font.Style:=Bf.Style;
        Font.Charset:=Bf.Charset;
        Bf.Free;
    End;
End;
//
// Paint the component
//
Procedure TPopupButton.Paint;
Begin
    Inherited Paint;
    With Canvas do begin
        If Focused then
            PaintButton(btFocus)
        Else
            PaintButton(btNormal);
        If MDown then
            PaintButton(btDown);
    End;
End;
//
// Message handlers
//
// Setfocus
Procedure TPopupButton.WMSetFocus(Var Message: TWMSetFocus);
Begin
   Inherited;
   Invalidate;
End;
// Kill focus
Procedure TPopupButton.WMKillFocus(Var Message: TWMKillFocus);
Begin
   Inherited;
   Invalidate;
End;
// Textchanged procedure
Procedure TPopupButton.CMTextChanged(var Msg: TMessage);
Begin
    Inherited;
    Invalidate;
End;
// Fontchanged procedure
Procedure TPopupButton.CMFontChanged(var Msg: TMessage);
Begin
    Inherited;
    Canvas.Font:=Font;
    Invalidate;
End;
// Enabled changes procedure
Procedure TPopupButton.CMEnabledChanged(var Msg: TMessage);
Begin
    Inherited;
    Invalidate;
End;
// Charset procedure
Procedure TPopupButton.CMDialogChar(var Msg: TCMDialogChar);
Var
    St: TShiftState;
Begin
    With Msg do
        If IsAccel(CharCode, Caption) and Enabled and CanFocus then
        Begin
            SetFocus;
            If Assigned(FPopupMenu) then
                MouseDown(mbLeft,St,0,0);
            End
        Else
            Inherited;
End;
// Notification procedure
Procedure TPopupButton.Notification(AComponent: TComponent; Operation: TOperation);
Begin
    Inherited Notification(AComponent, Operation);
    If (Operation=opRemove)  and (AComponent=FPopupMenu) then
        FPopupMenu := nil;
End;
// Click procedure
Procedure TPopupButton.Click;
Begin
     Inherited Click;
     mDown:=False;
     If Showing and CanFocus then
         SetFocus;
End;
//  Mousedown procedure
Procedure TPopupButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
Var
    Pt: Tpoint;
Begin
    SetFocus;
    Inherited MouseDown(Button,Shift,x,y);
    If Button=mbLeft then begin
        mDown:=True;
        Invalidate;
        Pt:=GetClientOrigin;
        If Assigned(PopupMenu) then
            PopupMenu.Popup(Pt.x,Pt.y+Height);
        mDown:=False;
        Invalidate;
    End;
End;
// Keypress procedure
Procedure TPopupButton.KeyPress(var Key: Char);
Var
    St: TShiftState;
Begin
    Inherited KeyPress(Key);
    If Key=' ' then
        MouseDown(mbLeft,St,0,0);
End;
End.
